--[[
	Name:		TargetArmor
	Version:	1.3
	Author:		Blackmail (knightchris2001@yahoo.com)
		Copyright (c) 2006 Christopher A. Florin (Blackmail on Arthus)
	Description:	Displays estimated armor value for targeted players and mobs.
	URL:		http://www.curse-gaming.com/mod.php?addid=3210
]]

local MELEEPERCENT = "MeleePercent";
local MELEEHITS = "MeleeHits";
local RANGEDPERCENT = "RangedPercent";
local RANGEDHITS = "RangedHits";

local configversion = 1;    --increment this any time the configuration format changes
local databaseversion = 2;  --increment this any time the database format changes

local AverageMeleeHit;
local AverageRangedHit;

local PlayerLevel = 1;
local PlayerName;

local TargetIsPlayer = true;
local TargetLevel = 70;
local TargetName = "Blackmail";

local lEventHandler = {};

local TargetArmor_DAMAGEDATA;

    if ( GetLocale() == "frFR" ) then
        TargetArmor_DAMAGEDATA =
            {
                { type = "Ranged", pattern = "Votre tire avec une arme \195\160 feu touche (.+) et inflige (%d+) points de d\195\169g\195\162ts.", target = 0, pts = 1 },
                { type = "Ranged", pattern = "Votre Tirer une fl\195\168che touche (.+) et inflige (%d+) points de d\195\169g\195\162ts.", target = 0, pts = 1 },
                { type = "Ranged", pattern = "Votre Tir \195\160 l\'arbal\195\168te touche (.+) et inflige (%d+) points de d\195\169g\195\162ts.", target = 0, pts = 1 },
    --            { type = "Ranged", pattern = "Your Auto Shot hits (.+) for (%d+).", target = 0, pts = 1 },
                { type = "Melee", pattern = "Vous touchez (.+) et infligez (%d+) points de d\195\169g\195\162ts.", target = 0, pts = 1 },

                { exit = true }
            };
    --elseif ( GetLocale()=="enUS" ) then
    else
        TargetArmor_DAMAGEDATA =
            {
                { type = "Ranged", pattern = "Your Shoot Bow hits (.+) for (%d+).", target = 0, pts = 1 },
                { type = "Ranged", pattern = "Your Shoot Crossbow hits (.+) for (%d+).", target = 0, pts = 1 },
                { type = "Ranged", pattern = "Your Shoot Gun hits (.+) for (%d+).", target = 0, pts = 1 },
                { type = "Ranged", pattern = "Your Auto Shot hits (.+) for (%d+).", target = 0, pts = 1 },
                { type = "Melee", pattern = "You hit (.+) for (%d+).", target = 0, pts = 1 },

                { exit = true }
            };
    end

RegisterForSave("MobArmor");
RegisterForSave("PlayerArmor");
RegisterForSave("TargetArmorConfig");

--OLDDATATHRESHOLD:
--  When TotalHitpointChange is equal to or greater than this number, half of all the previously collected data is
--      automatically assumed old and thrown out.
--  The default of 1000 is the equivalent of 1000% damage reduced. (total of all hits)
local OLDDATATHRESHOLD = 1000;
--PERCENTAGEMODIFIER:
--  The higher this number, an accurate reading will come faster, but it will adjust over time slower.
--  The lower this number, the slower an accurate reading will come, but it will adjust over time faster.
local PERCENTAGEMODIFIER = 0.98;

local function TargetArmor_Display()
    if ( not TargetName or TargetName == "" ) then HideUIPanel(TargetArmorFrame); return;
    else ShowUIPanel(TargetArmorFrame); end

    local DISPLAYTEXT;
    local lines;
    if ( TargetArmorConfig["showname"] == true ) then
        DISPLAYTEXT = "Level "..TargetLevel.." "..TargetName.." :\n"; lines = 1;
    else
        DISPLAYTEXT = ""; lines = 0;
    end

    local TotalRangedPercentage;
    local TotalRangedHits;
    local TotalMeleePercentage;
    local TotalMeleeHits;

    if ( TargetIsPlayer ) then
        if ( not PlayerArmor[PlayerName][TargetName] ) then
            DISPLAYTEXT = DISPLAYTEXT.."Invalid player name.";
            TargetArmor_SetDisplay(DISPLAYTEXT, lines+1);
            return;
        end

        TotalRangedPercentage = PlayerArmor[PlayerName][TargetName][RANGEDPERCENT];
        TotalRangedHits = PlayerArmor[PlayerName][TargetName][RANGEDHITS];
        TotalMeleePercentage = PlayerArmor[PlayerName][TargetName][MELEEPERCENT];
        TotalMeleeHits = PlayerArmor[PlayerName][TargetName][MELEEHITS];
    else
        local targetname = TargetName..":"..TargetLevel;

        if ( not MobArmor[PlayerName][targetname] ) then
            DISPLAYTEXT = DISPLAYTEXT.."Mob not found in database.\nPossible invalid name and/or level.";
            TargetArmor_SetDisplay(DISPLAYTEXT, lines+2);
            return;
        end

        TotalRangedPercentage = MobArmor[PlayerName][targetname][RANGEDPERCENT];
        TotalRangedHits = MobArmor[PlayerName][targetname][RANGEDHITS];
        TotalMeleePercentage = MobArmor[PlayerName][targetname][MELEEPERCENT];
        TotalMeleeHits = MobArmor[PlayerName][targetname][MELEEHITS];
    end

    if ( TotalRangedHits < 1 and TotalMeleeHits < 1 ) then
        DISPLAYTEXT = DISPLAYTEXT.."NO DATA.  PLEASE HIT THE TARGET FOR DATA.";
        TargetArmor_SetDisplay(DISPLAYTEXT, lines+2);
        return;
    end
    local pr = 0;

    if ( TotalMeleeHits > TotalRangedHits ) then
        pr = 1 - ( TotalMeleePercentage ) / ( TotalMeleeHits );
    else
        pr = 1 - (TotalRangedPercentage ) / ( TotalRangedHits );
    end

    local ea = TargetArmor_PercentToArmor(pr, PlayerLevel);

    pr = floor(pr*10000+.5)/100;
    ea = floor(ea+.5);

    DISPLAYTEXT = DISPLAYTEXT.."EA: "..ea.." / "..pr.."%";
    TargetArmor_SetDisplay(DISPLAYTEXT, lines+1);
end

function TargetArmor_SetDisplay(display, lines)
    TargetArmorFrame:SetHeight(15+(lines*10));
    TargetArmorText:SetHeight(15+(lines*10));
    TargetArmorText:SetText(display);
    if ( TargetArmorConfig["enabled"] == true ) then ShowUIPanel(TargetArmorFrame); end
end

--------------------------------------------------------------------------------------------------
-- Utility functions
--  * Neither of these functions attempt to do any rounding for you.
--------------------------------------------------------------------------------------------------
function TargetArmor_ArmorToPercent(armor, playerlevel)
    local ar = armor / ( 85 * playerlevel + 400 );
    return ar / ( ar + 1 );
end

function TargetArmor_PercentToArmor(percent, playerlevel)
    return ( percent * ( 85 * playerlevel + 400 ) ) / ( 1 - percent );
end

function TargetArmor_ResetConfig()
    TargetArmorConfig = {};

    TargetArmorConfig["version"] = configversion;

    TargetArmorConfig["bgcolor"] = {};
    TargetArmorConfig["bgcolor"]["red"] = 0.15;
    TargetArmorConfig["bgcolor"]["green"] = 0.15;
    TargetArmorConfig["bgcolor"]["blue"] = 0.15;
    TargetArmorConfig["bgcolor"]["alpha"] = 1.0;

    TargetArmorConfig["bordercolor"] = {};
    TargetArmorConfig["bordercolor"]["red"] = 0.4;
    TargetArmorConfig["bordercolor"]["green"] = 0.4;
    TargetArmorConfig["bordercolor"]["blue"] = 0.4;
    TargetArmorConfig["bordercolor"]["alpha"] = 1.0;

    TargetArmorConfig["textcolor"] = {};
    TargetArmorConfig["textcolor"]["red"] = 1.0;
    TargetArmorConfig["textcolor"]["green"] = 1.0;
    TargetArmorConfig["textcolor"]["blue"] = 1.0;
    TargetArmorConfig["textcolor"]["alpha"] = 1.0;

    TargetArmorConfig["enabled"] = true;
    TargetArmorConfig["showname"] = true;

    TargetArmor_UpdateDisplayConfigs();
end

--------------------------------------------------------------------------------------------------
-- OnFoo functions
--------------------------------------------------------------------------------------------------
function TargetArmor_OnLoad()
	for key,value in lEventHandler do
		this:RegisterEvent(key);
	end

    SlashCmdList["TargetArmor"] = function(arg1) TargetArmor_SlashCommandHandler(arg1); end
    SLASH_TargetArmor1 = "/targetarmor";   --/ta is used by tracker assist; besides abbreviations are too easy to overlap
end

function TargetArmor_OnEvent(event)
	if (lEventHandler[event]) then
		lEventHandler[event](arg1, arg2, arg3, arg4);
	end
end

function TargetArmor_SlashCommandHandler(arg1)
    local s, e, command;
    local args = {};
	s, e, command, args[0], args[1], args[2], args[3], args[4], args[5] = string.find(arg1, "(.+) (.+) (.+) (.+) (.+)");
	if ( not args[5] ) then s, e, command, args[0], args[1], args[2], args[3], args[4]  = string.find(arg1, "(.+) (.+) (.+) (.+) (.+) (.+)"); end
	if ( not args[4] ) then s, e, command, args[0], args[1], args[2], args[3] = string.find(arg1, "(.+) (.+) (.+) (.+) (.+)"); end
	if ( not args[3] ) then s, e, command, args[0], args[1], args[2] = string.find(arg1, "(.+) (.+) (.+) (.+)"); end
	if ( not args[2] ) then s, e, command, args[0], args[1] = string.find(arg1, "(.+) (.+) (.+)"); end
	if ( not args[1] ) then s, e, command, args[0] = string.find(arg1, "(.+) (.+)"); end
	if ( not args[0] ) then command = arg1; end

    if ( command ) then command = string.lower(command); else command = string.lower(arg1); end

    if ( command == "bgcolor" ) then
        if ( not args[0] and not args[1] ) then return; end
        if ( args[0] == "red" ) then    --yes, I'm aware this if statement could be rewritten better but its clear what's supposed to happen this way.
            TargetArmorConfig["bgcolor"]["red"] = args[1]/255;
        elseif ( args[0] == "green" ) then
            TargetArmorConfig["bgcolor"]["green"] = args[1]/255;
        elseif ( args[0] == "blue" ) then
            TargetArmorConfig["bgcolor"]["blue"] = args[1]/255;
        elseif ( args[0] == "alpha" ) then
            TargetArmorConfig["bgcolor"]["alpha"] = args[1]/255;
        else
            if ( not args[2] ) then return; end
            if ( not args[3] ) then args[3] = 1.0; end
            TargetArmorConfig["bgcolor"]["red"] = args[0]/255;
            TargetArmorConfig["bgcolor"]["green"] = args[1]/255;
            TargetArmorConfig["bgcolor"]["blue"] = args[2]/255;
            TargetArmorConfig["bgcolor"]["alpha"] = args[3]/255;
        end
    elseif ( command == "bordercolor" ) then
        if ( not args[0] and not args[1] ) then return; end
        if ( args[0] == "red" ) then    --yes, I'm aware this if statement could be rewritten better but its clear what's supposed to happen this way.
            TargetArmorConfig["bordercolor"]["red"] = args[1]/255;
        elseif ( args[0] == "green" ) then
            TargetArmorConfig["bordercolor"]["green"] = args[1]/255;
        elseif ( args[0] == "blue" ) then
            TargetArmorConfig["bordercolor"]["blue"] = args[1]/255;
        elseif ( args[0] == "alpha" ) then
            TargetArmorConfig["bordercolor"]["alpha"] = args[1]/255;
        else
            if ( not args[0] and not args[1] and not args[2] ) then return; end
            if ( not args[3] ) then args[3] = 1.0; end
            TargetArmorConfig["bordercolor"]["red"] = args[0]/255;
            TargetArmorConfig["bordercolor"]["green"] = args[1]/255;
            TargetArmorConfig["bordercolor"]["blue"] = args[2]/255;
            TargetArmorConfig["bordercolor"]["alpha"] = args[3]/255;
        end
    elseif ( command == "disable" ) then
        HideUIPanel(TargetArmorFrame);
        TargetArmorConfig["enabled"] = false;
    elseif ( command == "displayplayer" or command == "showplayer" ) then
        if ( not args[0] ) then return; end
        local playername = string.upper(string.sub(args[0], 1, 1))..string.lower(string.sub(args[0], 2));
        TargetName = playername; TargetLevel = -1; TargetIsPlayer = true;
        TargetArmor_Display();
    elseif ( command == "displaymob" or command == "showmob" ) then
        if ( not args[0] and not args[1] ) then return; end
        if ( args[3] ) then
            TargetName = args[0].." "..args[1].." "..args[2];
            TargetLevel = tonumber(args[3]);
        elseif ( args[2] ) then
            TargetName = args[0].." "..args[1];
            TargetLevel = tonumber(args[2]);
        else
            TargetName = args[0];
            TargetLevel = tonumber(args[1]);
        end
        TargetIsPlayer = false;
        TargetArmor_Display();
    elseif ( command == "enable" ) then
        ShowUIPanel(TargetArmorFrame);
        TargetArmorConfig["enabled"] = true;
    elseif ( command == "hidename" ) then
	    TargetArmorConfig["showname"] = false;
	    TargetArmor_Display();
    elseif ( command == "resetconfig" ) then
    	TargetArmor_ResetConfig();
    elseif ( command == "showname" ) then
	    TargetArmorConfig["showname"] = true;
	    TargetArmor_Display();
    elseif ( command == "textcolor" ) then
        if ( not args[0] and not args[1] ) then return; end
        if ( args[0] == "red" ) then    --yes, I'm aware this if statement could be rewritten better but its clear what's supposed to happen this way.
            TargetArmorConfig["textcolor"]["red"] = args[1]/255;
        elseif ( args[0] == "green" ) then
            TargetArmorConfig["textcolor"]["green"] = args[1]/255;
        elseif ( args[0] == "blue" ) then
            TargetArmorConfig["textcolor"]["blue"] = args[1]/255;
        elseif ( args[0] == "alpha" ) then
            TargetArmorConfig["textcolor"]["alpha"] = args[1]/255;
        else
            if ( not args[0] and not args[1] and not args[2] ) then return; end
            if ( not args[3] ) then args[3] = 1.0; end
            TargetArmorConfig["textcolor"]["red"] = args[0]/255;
            TargetArmorConfig["textcolor"]["green"] = args[1]/255;
            TargetArmorConfig["textcolor"]["blue"] = args[2]/255;
            TargetArmorConfig["textcolor"]["alpha"] = args[3]/255;
        end
    else
        DEFAULT_CHAT_FRAME:AddMessage("TargetArmor: Invalid command: "..command);
    end

    TargetArmor_UpdateDisplayConfigs();
end

function TargetArmor_UpdateDisplayConfigs()
    TargetArmorFrame:SetBackdropColor(TargetArmorConfig["bgcolor"]["red"], TargetArmorConfig["bgcolor"]["green"],
        TargetArmorConfig["bgcolor"]["blue"], TargetArmorConfig["bgcolor"]["alpha"]);

    TargetArmorFrame:SetBackdropBorderColor(TargetArmorConfig["bordercolor"]["red"], TargetArmorConfig["bordercolor"]["green"],
        TargetArmorConfig["bordercolor"]["blue"], TargetArmorConfig["bordercolor"]["alpha"]);

    TargetArmorText:SetTextColor(TargetArmorConfig["textcolor"]["red"], TargetArmorConfig["textcolor"]["green"],
        TargetArmorConfig["textcolor"]["blue"], TargetArmorConfig["textcolor"]["alpha"]);

    if ( TargetArmorConfig["enabled"] == true ) then    --default layout ni the xml code
--        ShowUIPanel(TargetArmorFrame);
    else
        HideUIPanel(TargetArmorFrame);
    end

--    if ( TargetArmorConfig["showname"] == true ) then
--    else
--    end
end

lEventHandler["PLAYER_LOGIN"] = function ()
    PlayerName = UnitName("player");
    PlayerLevel = UnitLevel("player");

    if ( not MobArmor[PlayerName] ) then MobArmor[PlayerName] = {}; end
    if ( not PlayerArmor[PlayerName] ) then PlayerArmor[PlayerName] = {}; end

    TargetLevel = PlayerLevel;
    TargetName = PlayerName;
    TargetIsPlayer = true;
    TargetArmor_Display();

    TargetArmor_UpdateDisplayConfigs();
end



lEventHandler["VARIABLES_LOADED"] = function()
	if ( not MobArmor ) then
	    MobArmor = {};
	    MobArmor["version"] = databaseversion;
    elseif ( MobArmor["version"] == 1 ) then
        for index, value in MobArmor do
            if ( index ~= "version" ) then
                for index2, value2 in MobArmor[index] do
                    MobArmor[index][index2][MELEEPERCENT] = MobArmor[index][index2].MELEEPERCENT;
                    MobArmor[index][index2][MELEEHITS] = MobArmor[index][index2].MELEEHITS;
                    MobArmor[index][index2][RANGEDPERCENT] = MobArmor[index][index2].RANGEDPERCENT;
                    MobArmor[index][index2][RANGEDHITS] = MobArmor[index][index2].RANGEDHITS;

                    MobArmor[index][index2].MELEEPERCENT = nil;
                    MobArmor[index][index2].MELEEHITS = nil;
                    MobArmor[index][index2].RANGEDPERCENT = nil;
                    MobArmor[index][index2].RANGEDHITS = nil;
                end
            end
        end
        MobArmor["version"] = 2;
	end

    if ( not PlayerArmor ) then
        PlayerArmor = {};
        PlayerArmor["version"] = databaseversion;
    elseif ( PlayerArmor["version"] == 1 ) then
        PlayerArmor["version"] = 2;
    end

	if ( not TargetArmorConfig ) then
	    TargetArmor_ResetConfig();
	end

end

lEventHandler["CHAT_MSG_COMBAT_SELF_HITS"] = function ()
    if ( not TargetName or TargetName == "" ) then return; end

    for index, value in TargetArmor_DAMAGEDATA do
        if ( value.exit ) then return; end

        local s, e;
        local results = { };
        s, e, results[0], results[1] = string.find(arg1, value.pattern);

        local match = false;
        if ( results[value.pts] and results[value.target] and results[value.target] == TargetName ) then match = true; end

        if ( match ) then
            if ( value.type == "Ranged" ) then
                --Do nothing; this is a melee event *SHOULD NOT HAPPEN*
            elseif ( value.type == "Melee" ) then
                local lowDmg, hiDmg, offlowDmg, offhiDmg, posBuff, negBuff, percentmod = UnitDamage("player");

                local currenthit = results[value.pts];  --Find the value of the last hit

                if ( ( offlowDmg ~= offhiDmg ) and ( offlowDmg ~= 0 or offhiDmg ~= 0 ) ) then
                    --If in here then the player is dualwielding.  We must not count the hits from offhand.
                    if ( tonumber(currenthit) < tonumber(offhiDmg) ) then return; end
                end

                AverageMeleeHit = (lowDmg + hiDmg) * percentmod / 2;    --Calculate the player's average melee hit
                local percentage = currenthit / AverageMeleeHit;

                local TotalMeleePercentage = 0;
                local TotalMeleeHits = 0;

                local OldDataModifier;

                --Retrieve old data
                if ( TargetIsPlayer ) then
                    TotalMeleePercentage = PlayerArmor[PlayerName][TargetName][MELEEPERCENT];
                    TotalMeleeHits = PlayerArmor[PlayerName][TargetName][MELEEHITS];
                    OldDataModifier = PERCENTAGEMODIFIER;
                else
                    local targetname = TargetName..":"..TargetLevel;

                    TotalMeleePercentage = MobArmor[PlayerName][targetname][MELEEPERCENT];
                    TotalMeleeHits = MobArmor[PlayerName][targetname][MELEEHITS];

                    OldDataModifier = PERCENTAGEMODIFIER;    --Mobs do not have evolving armor ratings, but players gain levels
                end

                if ( TotalMeleePercentage and TotalMeleeHits ) then   --If data exists, modify it
                    if ( TotalMeleePercentage > OLDDATATHRESHOLD ) then
                        TotalMeleePercentage = TotalMeleePercentage / 2;
                        TotalMeleeHits = TotalMeleeHits / 2;
                    end

                    TotalMeleePercentage = TotalMeleePercentage*OldDataModifier + percentage;
                    TotalMeleeHits = TotalMeleeHits*OldDataModifier + 1;
                else    --Set new data since no data exists
                    TotalMeleePercentage = percentage;
                    TotalMeleeHits = 1;
                end

                --Store new data
                if ( TargetIsPlayer ) then
                    PlayerArmor[PlayerName][TargetName][MELEEPERCENT] = TotalMeleePercentage;
                    PlayerArmor[PlayerName][TargetName][MELEEHITS] = TotalMeleeHits;
                else
                    local targetname = TargetName..":"..TargetLevel;

                    MobArmor[PlayerName][targetname][MELEEPERCENT] = TotalMeleePercentage;
                    MobArmor[PlayerName][targetname][MELEEHITS] = TotalMeleeHits;
                end

                --Calculate percentage decrease
                TargetArmor_Display();
                return;
            else
                --DEFAULT_CHAT_FRAME:AddMessage(value.type);
            end
        end
    end
end

lEventHandler["CHAT_MSG_SPELL_SELF_DAMAGE"] = function ()
    if ( not TargetName or TargetName == "" ) then return; end

    for index, value in TargetArmor_DAMAGEDATA do
        if ( value.exit ) then return; end

        local s, e;
        local results = { };
        s, e, results[0], results[1] = string.find(arg1, value.pattern);

        local match = false;
        if ( results[value.pts] and results[value.target] and results[value.target] == TargetName ) then match = true; end

        if ( match ) then
            if ( value.type == "Ranged" ) then
                local speed, lowDmg, hiDmg = UnitRangedDamage("player");    --Yes these two lines are ripped from WowWiki
                AverageRangedHit = (lowDmg + hiDmg) / 2;    --Calculate the player's average ranged hit

                local currenthit = results[value.pts];  --Find the value of the last hit
                local percentage = currenthit / AverageRangedHit;

                local TotalRangedPercentage = 0;
                local TotalRangedHits = 0;

                local OldDataModifier;

                --Retrieve old data
                if ( TargetIsPlayer ) then
                    TotalRangedPercentage = PlayerArmor[PlayerName][TargetName][RANGEDPERCENT];
                    TotalRangedHits = PlayerArmor[PlayerName][TargetName][RANGEDHITS];
                    OldDataModifier = PERCENTAGEMODIFIER;
                else
                    local targetname = TargetName..":"..TargetLevel;

                    TotalRangedPercentage = MobArmor[PlayerName][targetname][RANGEDPERCENT];
                    TotalRangedHits = MobArmor[PlayerName][targetname][RANGEDHITS];

                    OldDataModifier = PERCENTAGEMODIFIER;    --Mobs do not have evolving armor ratings, but players gain levels
                end

                if ( TotalRangedPercentage and TotalRangedHits ) then   --If data exists, modify it
                    if ( TotalRangedPercentage > OLDDATATHRESHOLD ) then
                        TotalRangedPercentage = TotalRangedPercentage / 2;
                        TotalRangedHits = TotalRangedHits / 2;
                    end

                    TotalRangedPercentage = TotalRangedPercentage*OldDataModifier + percentage;
                    TotalRangedHits = TotalRangedHits*OldDataModifier + 1;
                else    --Set new data since no data exists
                    TotalRangedPercentage = percentage;
                    TotalRangedHits = 1;
                end

                --Store new data
                if ( TargetIsPlayer ) then
                    PlayerArmor[PlayerName][TargetName][RANGEDPERCENT] = TotalRangedPercentage;
                    PlayerArmor[PlayerName][TargetName][RANGEDHITS] = TotalRangedHits;
                else
                    local targetname = TargetName..":"..TargetLevel;

                    MobArmor[PlayerName][targetname][RANGEDPERCENT] = TotalRangedPercentage;
                    MobArmor[PlayerName][targetname][RANGEDHITS] = TotalRangedHits;
                end

                --Calculate percentage decrease
                TargetArmor_Display();
                return;
            elseif ( value.type == "Melee" ) then
                --Do nothing, this is a Ranged event *SHOULD NOT HAPPEN*
            end
        end
    end
end

lEventHandler["PLAYER_TARGET_CHANGED"] = function ()
    TargetLevel = UnitLevel("target");
    TargetIsPlayer = UnitIsPlayer("target");
    TargetName = UnitName("target");

    if ( not TargetIsPlayer ) then TargetIsPlayer = false; end
    if ( not TargetName or TargetName == "") then return; end

    if ( TargetIsPlayer ) then

        if ( not PlayerArmor[PlayerName][TargetName] ) then
            PlayerArmor[PlayerName][TargetName] = {};
            PlayerArmor[PlayerName][TargetName][RANGEDPERCENT] = 0;
            PlayerArmor[PlayerName][TargetName][RANGEDHITS] = 0;
            PlayerArmor[PlayerName][TargetName][MELEEPERCENT] = 0;
            PlayerArmor[PlayerName][TargetName][MELEEHITS] = 0;
        end
    else
        local targetname = TargetName..":"..TargetLevel;

        if ( not MobArmor[PlayerName][targetname] ) then
            MobArmor[PlayerName][targetname] = {};
            MobArmor[PlayerName][targetname][MELEEPERCENT] = 0;
            MobArmor[PlayerName][targetname][MELEEHITS] = 0;
            MobArmor[PlayerName][targetname][RANGEDPERCENT] = 0;
            MobArmor[PlayerName][targetname][RANGEDHITS] = 0;
        else

        end
    end

     TargetArmor_Display();
end